--[[-------------------------------------------------------------------------
	SlashIn
    Copyright (C) 2010  Morsker
	Please contact Morsker through PM on forums.wowace.com.

	Provides the /in command for delayed execution.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
---------------------------------------------------------------------------]]
local addonName, addonScope = ...
local AceTimer30 = LibStub("AceTimer-3.0")

local raw_print = print
local tonumber = tonumber
local MacroEditBox = MacroEditBox
local MacroEditBox_OnEvent = MacroEditBox:GetScript("OnEvent")
-- GLOBALS: SLASH_SLASHIN_IN1, SLASH_SLASHIN_IN2, SlashCmdList

-- Print with colored prefix.
local printPrefix = "|cff33ff99"..addonName.."|r:"
local function print(...)
	raw_print(printPrefix, ...)
end

-- We execute lines by faking them as EXECUTE_CHAT_LINE events to the MacroEditBox defined in ChatFrame.lua.
-- The main benefit of this is that MacroEditBox gets special treatment in ChatEdit_OnEscapePressed.
-- It's also elegant, and reuses Blizzard code.
-- The main concern is taint, but I've tested it fairly well, and analyzed the execution path, and I'm
-- reasonably sure taint isn't an issue.
-- If taint does become a problem, there are other implementations that work just as well; they're just
-- less elegant. The dev version in the SVN has an alternative implementation commented out at the bottom.
local function OnCallback(command)
	MacroEditBox_OnEvent(MacroEditBox, "EXECUTE_CHAT_LINE", command)
end

local function SlashInCommand(msg)
	local secs, command = msg:match("^([^%s]+)%s+(.*)$")
	secs = tonumber(secs)
	if (not secs) or (#command == 0) then
		print("usage:\n /in <seconds> <command>")
		print("example:\n /in 1.5 /say hi")
	else
		AceTimer30.ScheduleTimer(addonScope, OnCallback, secs, command)
	end
end

SLASH_SLASHIN_IN1 = "/in"
SLASH_SLASHIN_IN2 = "/slashin"
SlashCmdList.SLASHIN_IN = SlashInCommand
